:- module(test, [run/0, run/1, run/3]).

:- use_module(library(lists)).
:- use_module(library(timeout)).
:- ensure_loaded(ksatrak).

mybetween(N, _M, N).
mybetween(N, M, X) :-
     N < M, N1 is N+1, mybetween(N1, M, X).

test_file_names(N, FileIn, FileCh) :-
	N1 is 100+N,
	number_codes(N1, [_|S0]),
	append("tests/test", S0, S1),
	append(S1, "d.txt", InS),
	append(S1, "p.txt", ChS),
	atom_codes(FileIn, InS),
	atom_codes(FileCh, ChS).

get_sols(FileCh, Sols) :-
	open(FileCh, read, S),
	read(S, Sols),
	close(S).

problem_data(satrak(SOk, OOk, Fak), S, O, F) :-
	length(SOk, S),
	length(OOk, O),
	length(Fak, F).

futtat(Problem, TimeOut, T) :-
	test_file_names(Problem, FileIn, FileCh),
	TimeOutMsec is TimeOut*1000,
	satrak_be(FileIn, Leiro),
	statistics(runtime, [T0|_]),
	time_out(
		 findall(X, user:satrak(Leiro, X), Sols),
		 TimeOutMsec, Res),
	statistics(runtime, [T1|_]),
	(   Res = success ->
	    sort(Sols, SSols),
	    length(Sols, Solno),
	    (   get_sols(FileCh, SSols) -> Status = 'OK   '
	    ;   Status = '!!ERR'
	    )
	;   Status = '!!TIM',
	    Solno = '??'
	),
	T is T1-T0,
	problem_data(Leiro, S, O, F),
	format('Feladat: ~t~d~11|, ~dx~d,~8+ fak szama: ~d,~t~17+ido: ~t~3d mp~15+,~t~w megoldas~13+, statusz: ~w\n',
	       [Problem, S, O, F, T, Solno, Status]).

run :-
	run(30).

run(TimeOut) :-
	run(1, 32, TimeOut).

run(Tol, Ig, TimeOut) :-
	bb_put(szumma_ido, 0),
	mybetween(Tol, Ig, N),
	futtat(N, TimeOut, T),
	bb_get(szumma_ido, T0),
	T1 is T0+T,
	bb_put(szumma_ido, T1),	
	fail.
run(_, _, _) :-
	bb_get(szumma_ido, T),
	format('~nOsszesen: ~t~3d mp~51|~n', [T]).

